@@ -81,6 +81,7 @@ import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.mail.api.IMailFolderStorageDelegator;
 import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.MailAccess;
+import com.openexchange.mail.api.MailCapabilities;
 import com.openexchange.mail.dataobjects.IDMailMessage;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.MessageHeaders;
@@ -195,7 +196,6 @@ public final class ImapIdlePushListener implements PushListener, Runnable {
     private final boolean supportsPermanentListeners;
     private volatile IMAPFolder imapFolderInUse;
     private volatile Map<String, Object> additionalProps;
-    private volatile long lastLockRefreshNanos;
     private volatile boolean interrupted;
     private volatile boolean idling;
 
@@ -215,7 +215,6 @@ public final class ImapIdlePushListener implements PushListener, Runnable {
         this.control = control;
         this.pushMode = pushMode;
         additionalProps = null;
-        lastLockRefreshNanos = System.nanoTime();
     }
 
     private boolean isUserValid() {
@@ -433,9 +432,7 @@ public final class ImapIdlePushListener implements PushListener, Runnable {
                     }
 
                     // Refresh lock prior to entering IMAP-IDLE
-                    if (doRefreshLock()) {
-                        ImapIdlePushManagerService.getInstance().refreshLock(new SessionInfo(session, permanent, permanent ? false : isTransient(session, services)));
-                    }
+                    ImapIdlePushManagerService.getInstance().refreshLock(new SessionInfo(session, permanent, permanent ? false : isTransient(session, services)));
 
                     // Are there already new messages?
                     {
@@ -575,21 +572,6 @@ public final class ImapIdlePushListener implements PushListener, Runnable {
         }
     }
 
-    /**
-     * Checks whether held cluster lock needs to be refreshed.
-     *
-     * @return <code>true</code> if refresh is needed; otherwise <code>false</code>
-     */
-    private boolean doRefreshLock() {
-        long last = lastLockRefreshNanos;
-        long nanos = System.nanoTime();
-        if (nanos - last > TimeUnit.MILLISECONDS.toNanos(TIMEOUT_THRESHOLD_MILLIS)) {
-            lastLockRefreshNanos = nanos;
-            return true;
-        }
-        return false;
-    }
-
     /**
      * Actually enters the IMAP-IDLE to IMAP server with respect to frequent cluster lock <i><tt>touch</tt></i>ing.
      * <p>
@@ -670,8 +652,8 @@ public final class ImapIdlePushListener implements PushListener, Runnable {
              * Check for IDLE capability
              */
             access.connect(false);
-            final IMAPCapabilities capabilities = (IMAPCapabilities) access.getMailConfig().getCapabilities();
-            if (!capabilities.hasIdle()) {
+            final MailCapabilities capabilities = access.getMailConfig().getCapabilities();
+            if (!(capabilities instanceof IMAPCapabilities) || !((IMAPCapabilities) capabilities).hasIdle()) {
                 throw PushExceptionCodes.UNEXPECTED_ERROR.create("Primary IMAP account does not support \"IDLE\" capability!");
             }
         } catch (OXException e) {
@@ -728,7 +710,6 @@ public final class ImapIdlePushListener implements PushListener, Runnable {
                 } else {
                     try {
                         anotherListener.start();
-                        cleanUpTask = null;
                     } catch (Exception e) {
                         LOGGER.warn("Failed to start new listener for user {} in context {}.", Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()), e);
                         // Give up lock and return
