@@ -51,7 +51,6 @@ package com.openexchange.push.imapidle;
 
 import static com.openexchange.java.Autoboxing.I;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.LinkedList;
@@ -85,7 +84,9 @@ import com.openexchange.push.PushUtility;
 import com.openexchange.push.imapidle.ImapIdlePushListener.PushMode;
 import com.openexchange.push.imapidle.control.ImapIdleControl;
 import com.openexchange.push.imapidle.control.ImapIdleControlTask;
+import com.openexchange.push.imapidle.control.ImapIdlePeriodicControlTask;
 import com.openexchange.push.imapidle.locking.ImapIdleClusterLock;
+import com.openexchange.push.imapidle.locking.ImapIdleClusterLock.AcquisitionResult;
 import com.openexchange.push.imapidle.locking.SessionInfo;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.server.ServiceLookup;
@@ -96,6 +97,8 @@ import com.openexchange.sessiond.SessiondServiceExtended;
 import com.openexchange.sessionstorage.hazelcast.serialization.PortableMultipleSessionRemoteLookUp;
 import com.openexchange.sessionstorage.hazelcast.serialization.PortableSession;
 import com.openexchange.sessionstorage.hazelcast.serialization.PortableSessionCollection;
+import com.openexchange.threadpool.AbstractTask;
+import com.openexchange.threadpool.ThreadPoolService;
 import com.openexchange.timer.ScheduledTimerTask;
 import com.openexchange.timer.TimerService;
 
@@ -126,7 +129,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
     }
 
     public static ImapIdlePushManagerService newInstance(ImapIdleConfiguration configuration, ServiceLookup services) throws OXException {
-        ImapIdlePushManagerService tmp = new ImapIdlePushManagerService(configuration.getFullName(), configuration.getAccountId(), configuration.getPushMode(), configuration.getDelay(), configuration.getClusterLock(), services);
+        ImapIdlePushManagerService tmp = new ImapIdlePushManagerService(configuration.getFullName(), configuration.getAccountId(), configuration.getPushMode(), configuration.getDelay(), configuration.getClusterLock(), configuration.isCheckPeriodic(), services);
         instance = tmp;
         return tmp;
     }
@@ -136,7 +139,12 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         return (service instanceof SessiondServiceExtended) ? false == ((SessiondServiceExtended) service).isApplicableForSessionStorage(session) : false;
     }
 
-    // ---------------------------------------------------------------------------------------------------- //
+    private static interface Cancelable {
+
+        void cancel();
+    }
+
+    // -------------------------------------------------------------------------------------------------------------------------------------
 
     private final String name;
     private final ServiceLookup services;
@@ -147,13 +155,13 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
     private final PushMode pushMode;
     private final ImapIdleClusterLock clusterLock;
     private final long delay;
-    private final ScheduledTimerTask timerTask;
+    private final Cancelable cancelable;
     private final AccessControl globalLock;
 
     /**
      * Initializes a new {@link ImapIdlePushManagerService}.
      */
-    private ImapIdlePushManagerService(String fullName, int accountId, PushMode pushMode, long delay, ImapIdleClusterLock clusterLock, ServiceLookup services) throws OXException {
+    private ImapIdlePushManagerService(String fullName, int accountId, PushMode pushMode, long delay, ImapIdleClusterLock clusterLock, boolean checkPeriodic, ServiceLookup services) throws OXException {
         super();
         name = "IMAP-IDLE Push Manager";
         this.pushMode = pushMode;
@@ -165,12 +173,37 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         this.control = new ImapIdleControl();
         listeners = new ConcurrentHashMap<SimpleKey, ImapIdlePushListener>(512, 0.9f, 1);
 
-        // Initialize timer task to check for expired IMAP-IDLE push listeners for every 30 seconds
-        TimerService timerService = services.getOptionalService(TimerService.class);
-        if (null == timerService) {
-            throw ServiceExceptionCode.absentService(TimerService.class);
+        if (checkPeriodic) {
+            // Initialize timer task to check for expired IMAP-IDLE push listeners for every 30 seconds
+            TimerService timerService = services.getOptionalService(TimerService.class);
+            if (null == timerService) {
+                throw ServiceExceptionCode.absentService(TimerService.class);
+            }
+            final ScheduledTimerTask timerTask = timerService.scheduleWithFixedDelay(new ImapIdlePeriodicControlTask(control), 30, 30, TimeUnit.SECONDS);
+            cancelable = new Cancelable() {
+
+                @Override
+                public void cancel() {
+                    timerTask.cancel();
+                }
+            };
+        } else {
+            // Initialize task to check for expired IMAP-IDLE push listeners
+            ThreadPoolService threadPool = services.getOptionalService(ThreadPoolService.class);
+            if (null == threadPool) {
+                throw ServiceExceptionCode.absentService(ThreadPoolService.class);
+            }
+            final ImapIdleControlTask controlTask = new ImapIdleControlTask(control);
+            final Future<Void> submitted = threadPool.submit(controlTask);
+            cancelable = new Cancelable() {
+
+                @Override
+                public void cancel() {
+                    controlTask.cancel();
+                    submitted.cancel(true);
+                }
+            };
         }
-        timerTask = timerService.scheduleWithFixedDelay(new ImapIdleControlTask(control), 30, 30, TimeUnit.SECONDS);
 
         // The fall-back lock
         globalLock = new ReentrantLockAccessControl();
@@ -194,7 +227,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
      * Shuts-down this IMAP-IDLE push manager.
      */
     public void shutDown() {
-        timerTask.cancel();
+        cancelable.cancel();
     }
 
     @Override
@@ -231,7 +264,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         return pushListenerService.generateSessionFor(pushUser);
     }
 
-    // ----------------------------------------------------------------------------------------------------------------------------------
+    // -------------------------------------------------------------------------------------------------------------------------------------
 
     @Override
     public List<PushUserInfo> getAvailablePushUsers() throws OXException {
@@ -261,8 +294,25 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         int userId = session.getUserId();
 
         SessionInfo sessionInfo = new SessionInfo(session, true, false);
-        if (clusterLock.acquireLock(sessionInfo)) {
+        AcquisitionResult acquisitionResult = clusterLock.acquireLock(sessionInfo);
+        if (AcquisitionResult.NOT_ACQUIRED == acquisitionResult) {
+            LOGGER.info("Could not acquire lock to start permanent IMAP-IDLE listener for user {} in context {} with session {} as there is already an associated listener", I(userId), I(contextId), session.getSessionID());
+        } else {
             // Locked...
+            switch (acquisitionResult) {
+                case ACQUIRED_NEW:
+                    LOGGER.info("Acquired lock to start permanent IMAP-IDLE listener for user {} in context {} with session {}", I(userId), I(contextId), session.getSessionID());
+                    break;
+                case ACQUIRED_NO_SUCH_SESSION:
+                    LOGGER.info("Acquired lock to start permanent IMAP-IDLE listener for user {} in context {} with session {} since the session is gone, which is associated with existent IMAP-IDLE listener", I(userId), I(contextId), session.getSessionID());
+                    break;
+                case ACQUIRED_TIMED_OUT:
+                    LOGGER.info("Acquired lock to start permanent IMAP-IDLE listener for user {} in context {} with session {} since lock for existent IMAP-IDLE listener timed out", I(userId), I(contextId), session.getSessionID());
+                    break;
+                default:
+                    break;
+            }
+
             boolean unlock = true;
             try {
                 AccessControl lock = getlockFor(userId, contextId);
@@ -298,8 +348,6 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
                     releaseLock(sessionInfo);
                 }
             }
-        } else {
-            LOGGER.info("Could not acquire lock to start IMAP-IDLE listener for user {} in context {} with session {} as there is already an associated listener", I(userId), I(contextId), session.getSessionID());
         }
 
         // No listener registered for given session
@@ -327,7 +375,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         return false;
     }
 
-    // ----------------------------------------------------------------------------------------------------------------------------------
+    // -------------------------------------------------------------------------------------------------------------------------------------
 
     @Override
     public PushListener startListener(Session session) throws OXException {
@@ -338,8 +386,25 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         int userId = session.getUserId();
 
         SessionInfo sessionInfo = new SessionInfo(session, false, isTransient(session, services));
-        if (clusterLock.acquireLock(sessionInfo)) {
+        AcquisitionResult acquisitionResult = clusterLock.acquireLock(sessionInfo);
+        if (AcquisitionResult.NOT_ACQUIRED == acquisitionResult) {
+            LOGGER.info("Could not acquire lock to start IMAP-IDLE listener for user {} in context {} with session {} as there is already an associated listener", I(userId), I(contextId), session.getSessionID());
+        } else {
             // Locked...
+            switch (acquisitionResult) {
+                case ACQUIRED_NEW:
+                    LOGGER.info("Acquired lock to start IMAP-IDLE listener for user {} in context {} with session {}", I(userId), I(contextId), session.getSessionID());
+                    break;
+                case ACQUIRED_NO_SUCH_SESSION:
+                    LOGGER.info("Acquired lock to start IMAP-IDLE listener for user {} in context {} with session {} since the session is gone, which is associated with existent IMAP-IDLE listener", I(userId), I(contextId), session.getSessionID());
+                    break;
+                case ACQUIRED_TIMED_OUT:
+                    LOGGER.info("Acquired lock to start IMAP-IDLE listener for user {} in context {} with session {} since lock for existent IMAP-IDLE listener timed out", I(userId), I(contextId), session.getSessionID());
+                    break;
+                default:
+                    break;
+            }
+
             boolean unlock = true;
             try {
                 AccessControl lock = getlockFor(userId, contextId);
@@ -366,8 +431,6 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
                     releaseLock(sessionInfo);
                 }
             }
-        } else {
-            LOGGER.info("Could not acquire lock to start IMAP-IDLE listener for user {} in context {} with session {} ({}) as there is already an associated listener", I(userId), I(contextId), session.getSessionID(), session.getClient());
         }
 
         // No listener registered for given session
@@ -500,11 +563,9 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
             // If we're running a node-local lock, there is no need to check for sessions at other nodes
             if (ImapIdleClusterLock.Type.LOCAL != clusterLock.getType()) {
                 // Look-up remote sessions, too, if possible
-                sessions = lookUpRemoteSessionsFor(userId, contextId);
-                for (Session session : sessions) {
-                    if (!oldSessionId.equals(session.getSessionID()) && PushUtility.allowedClient(session.getClient(), session, true)) {
-                        return injectAnotherListenerUsing(session, false).injectedPushListener;
-                    }
+                Session session = lookUpRemoteSessionFor(oldSession);
+                if (null != session) {
+                    return injectAnotherListenerUsing(session, false).injectedPushListener;
                 }
             }
         }
@@ -512,11 +573,11 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         return null;
     }
 
-    private Collection<Session> lookUpRemoteSessionsFor(int userId, int contextId) {
+    private Session lookUpRemoteSessionFor(Session oldSession) {
         HazelcastInstance hzInstance = services.getOptionalService(HazelcastInstance.class);
         ObfuscatorService obfuscatorService = services.getOptionalService(ObfuscatorService.class);
         if (null == hzInstance || null == obfuscatorService) {
-            return Collections.emptyList();
+            return null;
         }
 
         Cluster cluster = hzInstance.getCluster();
@@ -527,12 +588,15 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         // Determine other cluster members
         Set<Member> otherMembers = getOtherMembers(cluster.getMembers(), localMember);
         if (otherMembers.isEmpty()) {
-            return Collections.emptyList();
+            return null;
         }
 
+        int contextId = oldSession.getContextId();
+        int userId = oldSession.getUserId();
+
         IExecutorService executor = hzInstance.getExecutorService("default");
         Map<Member, Future<PortableSessionCollection>> futureMap = executor.submitToMembers(new PortableMultipleSessionRemoteLookUp(userId, contextId), otherMembers);
-        Collection<Session> userSessions = new LinkedList<>();
+        String oldSessionId = oldSession.getSessionID();
         for (Iterator<Entry<Member, Future<PortableSessionCollection>>> it = futureMap.entrySet().iterator(); it.hasNext();) {
             Future<PortableSessionCollection> future = it.next().getValue();
             // Check Future's return value
@@ -546,7 +610,10 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
                     if (null != portableSessions) {
                         for (PortableSession portableSession : portableSessions) {
                             portableSession.setPassword(obfuscatorService.unobfuscate(portableSession.getPassword()));
-                            userSessions.add(portableSession);
+                            if (!oldSessionId.equals(portableSession.getSessionID()) && PushUtility.allowedClient(portableSession.getClient(), portableSession, true)) {
+                                cancelRest(it);
+                                return portableSession;
+                            }
                         }
                     }
                 } catch (InterruptedException e) {
@@ -577,7 +644,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
                 }
             }
         }
-        return userSessions;
+        return null;
     }
 
     /**
@@ -600,12 +667,30 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
      *
      * @param future The {@code Future} to cancel
      */
-    private <V> void cancelFutureSafe(Future<V> future) {
+    static <V> void cancelFutureSafe(Future<V> future) {
         if (null != future) {
             try { future.cancel(true); } catch (Exception e) {/*Ignore*/}
         }
     }
 
+    private <V> void cancelRest(final Iterator<Entry<Member, Future<PortableSessionCollection>>> it) {
+        ThreadPoolService threadPool = services.getOptionalService(ThreadPoolService.class);
+        if (null != threadPool) {
+            AbstractTask<Void> task = new AbstractTask<Void>() {
+
+                @Override
+                public Void call() throws Exception {
+                    while (it.hasNext()) {
+                        Future<PortableSessionCollection> future = it.next().getValue();
+                        cancelFutureSafe(future);
+                    }
+                    return null;
+                }
+            };
+            threadPool.submit(task);
+        }
+    }
+
     /**
      * Tries to look-up another valid session and injects a new listener for it (discarding the existing one bound to given <code>oldSession</code>)
      *
@@ -636,7 +721,7 @@ public final class ImapIdlePushManagerService implements PushManagerExtendedServ
         }
     }
 
-    // -------------------------------------------------------------------------------------------------------------------------------
+    // -------------------------------------------------------------------------------------------------------------------------------------
 
     private static class InjectedImapIdlePushListener {
 
