@@ -58,6 +58,7 @@ import com.openexchange.lock.LockService;
 import com.openexchange.mail.service.MailService;
 import com.openexchange.mailaccount.MailAccountDeleteListener;
 import com.openexchange.osgi.HousekeepingActivator;
+import com.openexchange.pns.PushNotificationService;
 import com.openexchange.push.PushListenerService;
 import com.openexchange.push.imapidle.ImapIdleConfiguration;
 import com.openexchange.push.imapidle.ImapIdleDeleteListener;
@@ -108,6 +109,7 @@ public class ImapIdleActivator extends HousekeepingActivator {
             track(registeringTracker.getFilter(), registeringTracker);
             trackService(HazelcastInstance.class);
         }
+        trackService(PushNotificationService.class);
         trackService(SessionStorageService.class);
         trackService(LockService.class);
         openTrackers();
